@@ -27,6 +27,7 @@ from magnum.api.controllers import base
 from magnum.api.controllers import link
 from magnum.api.controllers.v1 import cluster_actions
 from magnum.api.controllers.v1 import collection
+from magnum.api.controllers.v1 import nodegroup
 from magnum.api.controllers.v1 import types
 from magnum.api import expose
 from magnum.api import utils as api_utils
@@ -173,6 +174,15 @@ class Cluster(base.APIBase):
     faults = wsme.wsattr(wtypes.DictType(wtypes.text, wtypes.text))
     """Fault info collected from the heat resources of this cluster"""
 
+    fixed_network = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed network name to attach to the Cluster"""
+
+    fixed_subnet = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed subnet name to attach to the Cluster"""
+
+    floating_ip_enabled = wsme.wsattr(types.boolean)
+    """Indicates whether created clusters should have a floating ip or not."""
+
     def __init__(self, **kwargs):
         super(Cluster, self).__init__()
         self.fields = []
@@ -182,6 +192,11 @@ class Cluster(base.APIBase):
                 continue
             self.fields.append(field)
             setattr(self, field, kwargs.get(field, wtypes.Unset))
+        nodegroup_fields = ['node_count', 'master_count',
+                            'node_addresses', 'master_addresses']
+        for field in nodegroup_fields:
+            self.fields.append(field)
+            setattr(self, field, kwargs.get(field, wtypes.Unset))
 
     @staticmethod
     def _convert_with_links(cluster, url, expand=True):
@@ -230,7 +245,10 @@ class Cluster(base.APIBase):
                      created_at=timeutils.utcnow(),
                      updated_at=timeutils.utcnow(),
                      coe_version=None,
-                     container_version=None)
+                     container_version=None,
+                     fixed_network=None,
+                     fixed_subnet=None,
+                     floating_ip_enabled=True)
         return cls._convert_with_links(sample, 'http://localhost:9511', expand)
 
 
@@ -329,6 +347,8 @@ class ClustersController(base.Controller):
                                                     sort_key=sort_key,
                                                     sort_dir=sort_dir)
 
+    nodegroups = nodegroup.NodeGroupController()
+
     @expose.expose(ClusterCollection, types.uuid, int, wtypes.text,
                    wtypes.text)
     def get_all(self, marker=None, limit=None, sort_key='id',
@@ -408,7 +428,7 @@ class ClustersController(base.Controller):
             # parameter, currently the design is allowing admin user to list
             # all clusters from all projects. But the all_tenants is one of
             # the condition to do project filter in DB API. And it's also used
-            # by periodic tasks. So the could be removed in the future and
+            # by periodic tasks. So this could be removed in the future and
             # a new parameter 'project_id' would be added so that admin user
             # can list clusters for a particular project.
             context.all_tenants = True
@@ -463,23 +483,20 @@ class ClustersController(base.Controller):
         if cluster.keypair is None:
             cluster.keypair = cluster_template.keypair_id
 
-        # If docker_volume_size is not present, use cluster_template value
-        if (cluster.docker_volume_size == wtypes.Unset or
-                not cluster.docker_volume_size):
-            cluster.docker_volume_size = cluster_template.docker_volume_size
-
         # If labels is not present, use cluster_template value
         if cluster.labels == wtypes.Unset:
             cluster.labels = cluster_template.labels
 
-        # If master_flavor_id is not present, use cluster_template value
-        if (cluster.master_flavor_id == wtypes.Unset or
-                not cluster.master_flavor_id):
-            cluster.master_flavor_id = cluster_template.master_flavor_id
+        # If floating_ip_enabled is not present, use cluster_template value
+        if cluster.floating_ip_enabled == wtypes.Unset:
+            cluster.floating_ip_enabled = cluster_template.floating_ip_enabled
 
-        # If flavor_id is not present, use cluster_template value
-        if cluster.flavor_id == wtypes.Unset or not cluster.flavor_id:
-            cluster.flavor_id = cluster_template.flavor_id
+        attributes = ["docker_volume_size", "master_flavor_id", "flavor_id",
+                      "fixed_network", "fixed_subnet"]
+        for attr in attributes:
+            if (getattr(cluster, attr) == wtypes.Unset or
+                    not getattr(cluster, attr)):
+                setattr(cluster, attr, getattr(cluster_template, attr))
 
         cluster_dict = cluster.as_dict()
 
@@ -499,9 +516,12 @@ class ClustersController(base.Controller):
         cluster_dict['coe_version'] = None
         cluster_dict['container_version'] = None
 
+        node_count = cluster_dict.pop('node_count')
+        master_count = cluster_dict.pop('master_count')
         new_cluster = objects.Cluster(context, **cluster_dict)
         new_cluster.uuid = uuid.uuid4()
         pecan.request.rpcapi.cluster_create_async(new_cluster,
+                                                  master_count, node_count,
                                                   cluster.create_timeout)
 
         return ClusterID(new_cluster.uuid)
@@ -516,8 +536,8 @@ class ClustersController(base.Controller):
         :param cluster_ident: UUID or logical name of a cluster.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster = self._patch(cluster_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(cluster)
+        cluster, node_count = self._patch(cluster_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count)
         return ClusterID(cluster.uuid)
 
     @base.Controller.api_version("1.3")  # noqa
@@ -531,12 +551,18 @@ class ClustersController(base.Controller):
         :param rollback: whether to rollback cluster on update failure.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster = self._patch(cluster_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(cluster, rollback)
+        cluster, node_count = self._patch(cluster_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  rollback)
         return ClusterID(cluster.uuid)
 
     def _patch(self, cluster_ident, patch):
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, "cluster:update_all_projects",
+                           action="cluster:update_all_projects")
+            context.all_tenants = True
+
         cluster = api_utils.get_resource('Cluster', cluster_ident)
         policy.enforce(context, 'cluster:update', cluster.as_dict(),
                        action='cluster:update')
@@ -547,22 +573,17 @@ class ClustersController(base.Controller):
         except api_utils.JSONPATCH_EXCEPTIONS as e:
             raise exception.PatchError(patch=patch, reason=e)
 
-        # Update only the fields that have changed
-        for field in objects.Cluster.fields:
-            try:
-                patch_val = getattr(new_cluster, field)
-            except AttributeError:
-                # Ignore fields that aren't exposed in the API
-                continue
-            if patch_val == wtypes.Unset:
-                patch_val = None
-            if cluster[field] != patch_val:
-                cluster[field] = patch_val
-
-        delta = cluster.obj_what_changed()
+        # NOTE(ttsiouts): magnum.objects.Cluster.node_count will be a
+        # property so we won't be able to store it in the object. So
+        # instead of object_what_changed compare the new and the old
+        # clusters.
+        delta = set()
+        for field in new_cluster.fields:
+            if getattr(cluster, field) != getattr(new_cluster, field):
+                delta.add(field)
 
         validation.validate_cluster_properties(delta)
-        return cluster
+        return cluster, new_cluster.node_count
 
     @expose.expose(None, types.uuid_or_name, status_code=204)
     def delete(self, cluster_ident):
