@@ -183,6 +183,10 @@ class InvalidSubnet(Invalid):
     message = _("Received invalid subnet %(subnet)s.")
 
 
+class InvalidVersion(Invalid):
+    message = _("Received invalid tag for %(tag)s.")
+
+
 class HTTPNotFound(ResourceNotFound):
     pass
 
@@ -220,6 +224,12 @@ class InvalidMAC(Invalid):
     message = _("Expected a MAC address but received %(mac)s.")
 
 
+class InvalidDNS(Invalid):
+    message = _(
+        "Expected a single dns address or comma separated dns list, "
+        "but received %(dns)s.")
+
+
 class ConfigInvalid(Invalid):
     message = _("Invalid configuration file. %(error_msg)s")
 
@@ -269,6 +279,12 @@ class OperationInProgress(Invalid):
                 "progress.")
 
 
+class VolumeTypeNotFound(ResourceNotFound):
+    """The code here changed to 400 according to the latest document."""
+    message = _("Valid volume type could not be found.")
+    code = 400
+
+
 class ImageNotFound(ResourceNotFound):
     """The code here changed to 400 according to the latest document."""
     message = _("Image %(image_id)s could not be found.")
@@ -327,6 +343,13 @@ class FlavorNotFound(ResourceNotFound):
     code = 400
 
 
+class FixedNetworkNotFound(ResourceNotFound):
+    """The code here changed to 400 according to the latest document."""
+    """"Ensure the network is private."""
+    message = _("Unable to find fixed network %(network)s.")
+    code = 400
+
+
 class ExternalNetworkNotFound(ResourceNotFound):
     """The code here changed to 400 according to the latest document."""
     """"Ensure the network is not private."""
@@ -397,3 +420,22 @@ class MemberAlreadyExists(Conflict):
 class PreDeletionFailed(Conflict):
     message = _("Failed to pre-delete resources for cluster %(cluster_uuid)s, "
                 "error: %(msg)s.")
+
+
+class NodeGroupAlreadyExists(Conflict):
+    message = _("A node group with name %(name)s already exists in the "
+                "cluster %(cluster_id)s.")
+
+
+class NodeGroupNotFound(ResourceNotFound):
+    message = _("Nodegroup %(nodegroup)s could not be found.")
+
+
+class MasterNGResizeNotSupported(NotSupported):
+    message = _("Resizing a master nodegroup is not supported.")
+
+
+class NGResizeOutBounds(Invalid):
+    message = _("Resizing %(nodegroup)s outside the allowed range: "
+                "min_node_count = %(min_node_count)s, "
+                "max_node_count = %(max_node_count)s")
