@@ -1,18 +1,23 @@
 #!/bin/bash
 
-. /etc/sysconfig/heat-params
+set -x
+set +u
+HTTP_PROXY="$HTTP_PROXY"
+HTTPS_PROXY="$HTTPS_PROXY"
+NO_PROXY="$NO_PROXY"
+CONTAINER_INFRA_PREFIX="$CONTAINER_INFRA_PREFIX"
+HEAT_CONTAINER_AGENT_TAG="$HEAT_CONTAINER_AGENT_TAG"
 
-set -uxe
 
-if [ ! -z "$HTTP_PROXY" ]; then
+if [ -n "${HTTP_PROXY}" ]; then
     export HTTP_PROXY
 fi
 
-if [ ! -z "$HTTPS_PROXY" ]; then
+if [ -n "${HTTPS_PROXY}" ]; then
     export HTTPS_PROXY
 fi
 
-if [ ! -z "$NO_PROXY" ]; then
+if [ -n "${NO_PROXY}" ]; then
     export NO_PROXY
 fi
 
@@ -21,7 +26,7 @@ fi
 # in host mount namespace and apply configuration.
 mkdir -p /srv/magnum/.ssh
 chmod 700 /srv/magnum/.ssh
-ssh-keygen -t rsa -N '' -f /srv/magnum/.ssh/heat_agent_rsa
+ssh-keygen -q -t rsa -N '' -f /srv/magnum/.ssh/heat_agent_rsa
 chmod 400 /srv/magnum/.ssh/heat_agent_rsa
 chmod 400 /srv/magnum/.ssh/heat_agent_rsa.pub
 # Add the public to the host authorized_keys file.
@@ -38,16 +43,19 @@ Host localhost
 EOF
 
 sed -i '/^PermitRootLogin/ s/ .*/ without-password/' /etc/ssh/sshd_config
+# Security enhancement: Disable password authentication
+sed -i '/^PasswordAuthentication yes/ s/ yes/ no/' /etc/ssh/sshd_config
+
 systemctl restart sshd
 
 
-_prefix=${CONTAINER_INFRA_PREFIX:-docker.io/openstackmagnum/}
+_prefix="${CONTAINER_INFRA_PREFIX:-docker.io/openstackmagnum/}"
 atomic install \
 --storage ostree \
 --system \
 --system-package no \
 --set REQUESTS_CA_BUNDLE=/etc/pki/tls/certs/ca-bundle.crt \
 --name heat-container-agent \
-${_prefix}heat-container-agent:${HEAT_CONTAINER_AGENT_TAG}
+"${_prefix}heat-container-agent:${HEAT_CONTAINER_AGENT_TAG}"
 
 systemctl start heat-container-agent
