@@ -40,7 +40,7 @@ class TestRootController(api_base.FunctionalTest):
                                [{u'href': u'http://localhost/v1/',
                                  u'rel': u'self'}],
                            u'status': u'CURRENT',
-                           u'max_version': u'1.6',
+                           u'max_version': u'1.8',
                            u'min_version': u'1.1'}]}
 
         self.v1_expected = {
@@ -90,7 +90,13 @@ class TestRootController(api_base.FunctionalTest):
             u'federations': [{u'href': u'http://localhost/v1/federations/',
                               u'rel': u'self'},
                              {u'href': u'http://localhost/federations/',
-                              u'rel': u'bookmark'}]}
+                              u'rel': u'bookmark'}],
+            u'nodegroups': [{u'href': u'http://localhost/v1/clusters/'
+                                      '{cluster_id}/nodegroups',
+                             u'rel': u'self'},
+                            {u'href': u'http://localhost/clusters/'
+                                      '{cluster_id}/nodegroups',
+                             u'rel': u'bookmark'}]}
 
     def make_app(self, paste_file):
         file_name = self.get_path(paste_file)
