@@ -280,7 +280,11 @@ class TestPatch(api_base.FunctionalTest):
         self.mock_cluster_update.side_effect = self._sim_rpc_cluster_update
         self.addCleanup(p.stop)
 
-    def _sim_rpc_cluster_update(self, cluster, rollback=False):
+    def _sim_rpc_cluster_update(self, cluster, node_count, rollback=False):
+        cluster.status = 'UPDATE_IN_PROGRESS'
+        default_ng_worker = cluster.default_ng_worker
+        default_ng_worker.node_count = node_count
+        default_ng_worker.save()
         cluster.save()
         return cluster
 
@@ -437,23 +441,27 @@ class TestPatch(api_base.FunctionalTest):
         self.assertTrue(response.json['errors'])
 
     def test_update_cluster_with_rollback_enabled(self):
+        node_count = 4
         response = self.patch_json(
             '/clusters/%s/?rollback=True' % self.cluster_obj.uuid,
-            [{'path': '/node_count', 'value': 4,
+            [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
-        self.mock_cluster_update.assert_called_once_with(mock.ANY, True)
+        self.mock_cluster_update.assert_called_once_with(
+            mock.ANY, node_count, True)
         self.assertEqual(202, response.status_code)
 
     def test_update_cluster_with_rollback_disabled(self):
+        node_count = 4
         response = self.patch_json(
             '/clusters/%s/?rollback=False' % self.cluster_obj.uuid,
-            [{'path': '/node_count', 'value': 4,
+            [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
-        self.mock_cluster_update.assert_called_once_with(mock.ANY, False)
+        self.mock_cluster_update.assert_called_once_with(
+            mock.ANY, node_count, False)
         self.assertEqual(202, response.status_code)
 
     def test_remove_ok(self):
@@ -496,6 +504,19 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual(400, response.status_code)
         self.assertTrue(response.json['errors'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_update_cluster_as_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster(self.context, uuid=temp_uuid)
+        self.context.is_admin = True
+        response = self.patch_json('/clusters/%s' % temp_uuid,
+                                   [{'path': '/node_count',
+                                     'value': 4,
+                                     'op': 'replace'}])
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_code)
+
 
 class TestPost(api_base.FunctionalTest):
     def setUp(self):
@@ -510,7 +531,8 @@ class TestPost(api_base.FunctionalTest):
         self.mock_valid_os_res = p.start()
         self.addCleanup(p.stop)
 
-    def _simulate_cluster_create(self, cluster, create_timeout):
+    def _simulate_cluster_create(self, cluster, master_count, node_count,
+                                 create_timeout):
         cluster.create()
         return cluster
 
@@ -549,7 +571,8 @@ class TestPost(api_base.FunctionalTest):
     def test_create_cluster_set_project_id_and_user_id(self):
         bdict = apiutils.cluster_post_data()
 
-        def _simulate_rpc_cluster_create(cluster, create_timeout):
+        def _simulate_rpc_cluster_create(cluster, master_count, node_count,
+                                         create_timeout):
             self.assertEqual(self.context.project_id, cluster.project_id)
             self.assertEqual(self.context.user_id, cluster.user_id)
             cluster.create()
@@ -681,7 +704,8 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(202, response.status_int)
 
     def test_create_cluster_with_no_timeout(self):
-        def _simulate_rpc_cluster_create(cluster, create_timeout):
+        def _simulate_rpc_cluster_create(cluster, master_count, node_count,
+                                         create_timeout):
             self.assertEqual(60, create_timeout)
             cluster.create()
             return cluster
@@ -1010,6 +1034,9 @@ class TestClusterPolicyEnforcement(api_base.FunctionalTest):
     def _simulate_cluster_delete(self, cluster_uuid):
         cluster = objects.Cluster.get_by_uuid(self.context, cluster_uuid)
         cluster.destroy()
+        ngs = objects.NodeGroup.list(self.context, cluster_uuid)
+        for ng in ngs:
+            ng.destroy()
 
     def test_policy_disallow_delete(self):
         p = mock.patch.object(rpcapi.API, 'cluster_delete')
