@@ -27,6 +27,15 @@ CONF = magnum.conf.CONF
 class TestClusterConductorWithK8s(base.TestCase):
     def setUp(self):
         super(TestClusterConductorWithK8s, self).setUp()
+        self.keystone_auth_default_policy = ('[{"match": [{"type": "role", '
+                                             '"values": ["member"]}, {"type": '
+                                             '"project", "values": '
+                                             '["project_id"]}], "resource": '
+                                             '{"namespace": "default", '
+                                             '"resources": ["pods", '
+                                             '"services", "deployments", '
+                                             '"pvc"], "verbs": ["list"], '
+                                             '"version": "*"}}]')
         self.cluster_template_dict = {
             'image_id': 'image_id',
             'flavor_id': 'flavor_id',
@@ -75,13 +84,9 @@ class TestClusterConductorWithK8s(base.TestCase):
             'name': 'cluster1',
             'stack_id': 'xx-xx-xx-xx',
             'api_address': '172.17.2.3',
-            'node_addresses': ['172.17.2.4'],
-            'node_count': 1,
-            'master_count': 1,
             'discovery_url': 'https://discovery.etcd.io/test',
             'docker_volume_size': 20,
             'flavor_id': 'flavor_id',
-            'master_addresses': ['172.17.2.18'],
             'ca_cert_ref': 'http://barbican/v1/containers/xx-xx-xx-xx',
             'magnum_cert_ref': 'http://barbican/v1/containers/xx-xx-xx-xx',
             'trustee_username': 'fake_trustee',
@@ -109,10 +114,47 @@ class TestClusterConductorWithK8s(base.TestCase):
                        'kubescheduler_options': '--kubescheduler',
                        'kubeproxy_options': '--kubeproxy',
                        'influx_grafana_dashboard_enabled': 'True',
-                       'service_cluster_ip_range': '10.254.0.0/16'},
+                       'service_cluster_ip_range': '10.254.0.0/16',
+                       'boot_volume_size': '60'},
             'master_flavor_id': 'master_flavor_id',
             'flavor_id': 'flavor_id',
             'project_id': 'project_id',
+            'keystone_auth_default_policy': self.keystone_auth_default_policy,
+            'fixed_network': 'fixed_network',
+            'fixed_subnet': 'fixed_subnet',
+            'floating_ip_enabled': False,
+        }
+        self.worker_ng_dict = {
+            'uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
+            'name': 'worker_ng',
+            'cluster_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
+            'project_id': 'project_id',
+            'docker_volume_size': 20,
+            'labels': self.cluster_dict['labels'],
+            'flavor_id': 'flavor_id',
+            'image_id': 'image_id',
+            'node_addresses': ['172.17.2.4'],
+            'node_count': 1,
+            'role': 'worker',
+            'max_nodes': 5,
+            'min_nodes': 1,
+            'is_default': True
+        }
+        self.master_ng_dict = {
+            'uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a54',
+            'name': 'master_ng',
+            'cluster_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
+            'project_id': 'project_id',
+            'docker_volume_size': 20,
+            'labels': self.cluster_dict['labels'],
+            'flavor_id': 'master_flavor_id',
+            'image_id': 'image_id',
+            'node_addresses': ['172.17.2.18'],
+            'node_count': 1,
+            'role': 'master',
+            'max_nodes': 5,
+            'min_nodes': 1,
+            'is_default': True
         }
         self.context.user_name = 'fake_user'
         self.context.project_id = 'fake_tenant'
@@ -134,9 +176,14 @@ class TestClusterConductorWithK8s(base.TestCase):
         self.mock_enable_octavia = octavia_patcher.start()
         self.mock_enable_octavia.return_value = False
         self.addCleanup(octavia_patcher.stop)
+        CONF.set_override('default_boot_volume_type',
+                          'lvmdriver-1', group='cinder')
+        CONF.set_override('default_etcd_volume_type',
+                          'lvmdriver-1', group='cinder')
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -146,11 +193,13 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         self._test_extract_template_definition(
             mock_generate_csr_and_key, mock_sign_node_certificate,
-            mock_driver, mock_objects_cluster_template_get_by_uuid, mock_get)
+            mock_driver, mock_objects_cluster_template_get_by_uuid, mock_get,
+            mock_objects_nodegroup_list)
 
     def _test_extract_template_definition(
             self,
@@ -159,11 +208,16 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_driver,
             mock_objects_cluster_template_get_by_uuid,
             mock_get,
+            mock_objects_nodegroup_list,
             missing_attr=None):
         if missing_attr in self.cluster_template_dict:
             self.cluster_template_dict[missing_attr] = None
         elif missing_attr in self.cluster_dict:
             self.cluster_dict[missing_attr] = None
+        elif missing_attr == 'node_count':
+            self.worker_ng_dict['node_count'] = None
+        elif missing_attr == 'master_count':
+            self.master_ng_dict['node_count'] = None
         cluster_template = objects.ClusterTemplate(
             self.context, **self.cluster_template_dict)
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
@@ -179,6 +233,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         mock_resp.status_code = 200
         mock_get.return_value = mock_resp
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = k8s_dr.Driver()
 
         (template_path,
@@ -210,6 +267,8 @@ class TestClusterConductorWithK8s(base.TestCase):
                        'kube_dashboard_enabled': 'True',
                        'influx_grafana_dashboard_enabled': 'True',
                        'docker_volume_type': 'lvmdriver-1',
+                       'boot_volume_type': 'lvmdriver-1',
+                       'etcd_volume_type': 'lvmdriver-1',
                        'etcd_volume_size': None,
                        'availability_zone': 'az_1',
                        'cert_manager_api': 'False',
@@ -231,9 +290,11 @@ class TestClusterConductorWithK8s(base.TestCase):
             'insecure_registry': '10.0.0.1:5000',
         }
         expected = {
+            'cloud_provider_enabled': 'false',
             'ssh_key_name': 'keypair_id',
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'network_driver': 'network_driver',
             'volume_driver': 'volume_driver',
@@ -292,11 +353,25 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
             'portal_network_cidr': '10.254.0.0/16',
-            'project_id': 'project_id'
+            'project_id': 'project_id',
+            'max_node_count': 2,
+            'master_image': 'image_id',
+            'minion_image': 'image_id',
+            'keystone_auth_default_policy': self.keystone_auth_default_policy,
+            'boot_volume_size': '60',
+            'boot_volume_type': 'lvmdriver-1',
+            'etcd_volume_type': 'lvmdriver-1'
         }
         if missing_attr is not None:
             expected.pop(mapping[missing_attr], None)
 
+        if missing_attr == 'node_count':
+            expected['max_node_count'] = None
+
+        if missing_attr == 'image_id':
+            expected['master_image'] = None
+            expected['minion_image'] = None
+
         self.assertEqual(expected, definition)
         self.assertEqual(
             ['../../common/templates/environments/no_private_network.yaml',
@@ -310,6 +385,7 @@ class TestClusterConductorWithK8s(base.TestCase):
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -319,6 +395,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         self.cluster_template_dict['registry_enabled'] = True
@@ -336,6 +413,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         mock_resp.text = expected_result
         mock_get.return_value = mock_resp
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = k8s_dr.Driver()
 
         CONF.set_override('swift_region',
@@ -353,6 +433,7 @@ class TestClusterConductorWithK8s(base.TestCase):
 
         expected = {
             'auth_url': 'http://192.168.10.10:5000/v3',
+            'cloud_provider_enabled': 'true',
             'cluster_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
             'discovery_url': 'https://discovery.etcd.io/test',
             'dns_nameserver': 'dns_nameserver',
@@ -362,6 +443,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'etcd_volume_size': None,
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'flannel_backend': 'vxlan',
             'flannel_network_cidr': '10.101.0.0/16',
@@ -415,7 +497,14 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
             'portal_network_cidr': '10.254.0.0/16',
-            'project_id': 'project_id'
+            'project_id': 'project_id',
+            'max_node_count': 2,
+            'master_image': 'image_id',
+            'minion_image': 'image_id',
+            'keystone_auth_default_policy': self.keystone_auth_default_policy,
+            'boot_volume_size': '60',
+            'boot_volume_type': 'lvmdriver-1',
+            'etcd_volume_type': 'lvmdriver-1'
         }
 
         self.assertEqual(expected, definition)
@@ -431,6 +520,7 @@ class TestClusterConductorWithK8s(base.TestCase):
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -440,11 +530,12 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
 
         not_required = ['image_id', 'flavor_id', 'dns_nameserver',
-                        'docker_volume_size', 'fixed_network', 'http_proxy',
+                        'docker_volume_size', 'http_proxy',
                         'https_proxy', 'no_proxy', 'network_driver',
                         'master_flavor_id', 'docker_storage_driver',
                         'volume_driver', 'fixed_subnet']
@@ -467,6 +558,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         mock_get.return_value = mock_resp
         mock_driver.return_value = k8s_dr.Driver()
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
 
         (template_path,
          definition,
@@ -475,11 +569,15 @@ class TestClusterConductorWithK8s(base.TestCase):
 
         expected = {
             'auth_url': 'http://192.168.10.10:5000/v3',
+            'cloud_provider_enabled': 'false',
             'cluster_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
             'discovery_url': 'https://discovery.etcd.io/test',
             'docker_volume_size': 20,
             'master_flavor': 'master_flavor_id',
             'minion_flavor': 'flavor_id',
+            'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
+            'fixed_subnet': 'fixed_subnet',
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'flannel_backend': 'vxlan',
             'flannel_network_cidr': '10.101.0.0/16',
@@ -525,11 +623,18 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
             'portal_network_cidr': '10.254.0.0/16',
-            'project_id': 'project_id'
+            'project_id': 'project_id',
+            'max_node_count': 2,
+            'master_image': None,
+            'minion_image': None,
+            'keystone_auth_default_policy': self.keystone_auth_default_policy,
+            'boot_volume_size': '60',
+            'boot_volume_type': 'lvmdriver-1',
+            'etcd_volume_type': 'lvmdriver-1'
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
-            ['../../common/templates/environments/with_private_network.yaml',
+            ['../../common/templates/environments/no_private_network.yaml',
              '../../common/templates/environments/no_etcd_volume.yaml',
              '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
@@ -540,10 +645,12 @@ class TestClusterConductorWithK8s(base.TestCase):
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     def test_extract_template_definition_coreos_with_disovery(
             self,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         self.cluster_template_dict['cluster_distro'] = 'coreos'
@@ -558,6 +665,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         mock_resp.status_code = 200
         mock_get.return_value = mock_resp
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = k8s_coreos_dr.Driver()
 
         (template_path,
@@ -569,6 +679,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'ssh_key_name': 'keypair_id',
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'availability_zone': 'az_1',
             'nodes_affinity_policy': 'soft-anti-affinity',
@@ -640,10 +751,12 @@ class TestClusterConductorWithK8s(base.TestCase):
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     def test_extract_template_definition_coreos_no_discoveryurl(
             self,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             reqget):
         self.cluster_template_dict['cluster_distro'] = 'coreos'
@@ -656,6 +769,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = k8s_coreos_dr.Driver()
 
         (template_path,
@@ -668,6 +784,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'availability_zone': 'az_1',
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'dns_nameserver': 'dns_nameserver',
             'docker_storage_driver': u'devicemapper',
@@ -738,6 +855,7 @@ class TestClusterConductorWithK8s(base.TestCase):
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -747,6 +865,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         mock_driver.return_value = k8s_dr.Driver()
@@ -756,10 +875,12 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_driver,
             mock_objects_cluster_template_get_by_uuid,
             mock_get,
+            mock_objects_nodegroup_list,
             missing_attr='dns_nameserver')
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -769,6 +890,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         mock_driver.return_value = k8s_dr.Driver()
@@ -778,10 +900,12 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_driver,
             mock_objects_cluster_template_get_by_uuid,
             mock_get,
+            mock_objects_nodegroup_list,
             missing_attr='image_id')
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -791,6 +915,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         mock_driver.return_value = k8s_dr.Driver()
@@ -800,10 +925,12 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_driver,
             mock_objects_cluster_template_get_by_uuid,
             mock_get,
+            mock_objects_nodegroup_list,
             missing_attr='docker_storage_driver')
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -813,6 +940,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             mock_get):
         mock_driver.return_value = k8s_dr.Driver()
@@ -822,54 +950,12 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_driver,
             mock_objects_cluster_template_get_by_uuid,
             mock_get,
+            mock_objects_nodegroup_list,
             missing_attr='apiserver_port')
 
     @patch('requests.get')
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
-    @patch('magnum.drivers.common.driver.Driver.get_driver')
-    @patch('magnum.conductor.handlers.common.cert_manager'
-           '.sign_node_certificate')
-    @patch('magnum.common.x509.operations.generate_csr_and_key')
-    def test_extract_template_definition_without_node_count(
-            self,
-            mock_generate_csr_and_key,
-            mock_sign_node_certificate,
-            mock_driver,
-            mock_objects_cluster_template_get_by_uuid,
-            mock_get):
-        mock_driver.return_value = k8s_dr.Driver()
-        self._test_extract_template_definition(
-            mock_generate_csr_and_key,
-            mock_sign_node_certificate,
-            mock_driver,
-            mock_objects_cluster_template_get_by_uuid,
-            mock_get,
-            missing_attr='node_count')
-
-    @patch('requests.get')
-    @patch('magnum.objects.ClusterTemplate.get_by_uuid')
-    @patch('magnum.drivers.common.driver.Driver.get_driver')
-    @patch('magnum.conductor.handlers.common.cert_manager'
-           '.sign_node_certificate')
-    @patch('magnum.common.x509.operations.generate_csr_and_key')
-    def test_extract_template_definition_without_master_count(
-            self,
-            mock_generate_csr_and_key,
-            mock_sign_node_certificate,
-            mock_driver,
-            mock_objects_cluster_template_get_by_uuid,
-            mock_get):
-        mock_driver.return_value = k8s_dr.Driver()
-        self._test_extract_template_definition(
-            mock_generate_csr_and_key,
-            mock_sign_node_certificate,
-            mock_driver,
-            mock_objects_cluster_template_get_by_uuid,
-            mock_get,
-            missing_attr='master_count')
-
-    @patch('requests.get')
-    @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.conductor.handlers.common.cert_manager'
            '.sign_node_certificate')
@@ -879,6 +965,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             mock_generate_csr_and_key,
             mock_sign_node_certificate,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid,
             reqget):
         cluster_template = objects.ClusterTemplate(
@@ -892,6 +979,9 @@ class TestClusterConductorWithK8s(base.TestCase):
         cluster_dict = self.cluster_dict
         cluster_dict['discovery_url'] = None
         cluster = objects.Cluster(self.context, **cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = k8s_dr.Driver()
 
         CONF.set_override('etcd_discovery_service_endpoint_format',
@@ -907,9 +997,11 @@ class TestClusterConductorWithK8s(base.TestCase):
                                                                  cluster)
 
         expected = {
+            'cloud_provider_enabled': 'false',
             'ssh_key_name': 'keypair_id',
             'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
+            'fixed_network_name': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'dns_nameserver': 'dns_nameserver',
             'server_image': 'image_id',
@@ -968,7 +1060,14 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
             'portal_network_cidr': '10.254.0.0/16',
-            'project_id': 'project_id'
+            'project_id': 'project_id',
+            'max_node_count': 2,
+            'master_image': 'image_id',
+            'minion_image': 'image_id',
+            'keystone_auth_default_policy': self.keystone_auth_default_policy,
+            'boot_volume_size': '60',
+            'boot_volume_type': 'lvmdriver-1',
+            'etcd_volume_type': 'lvmdriver-1'
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
@@ -980,8 +1079,7 @@ class TestClusterConductorWithK8s(base.TestCase):
              '../../common/templates/environments/disable_lb_floating_ip.yaml',
              ],
             env_files)
-        reqget.assert_called_once_with('http://etcd/test?size=1', proxies={
-            'http': 'http_proxy', 'https': 'https_proxy'})
+        reqget.assert_called_once_with('http://etcd/test?size=1')
 
     @patch('magnum.common.short_id.generate_id')
     @patch('heatclient.common.template_utils.get_template_contents')
@@ -1109,21 +1207,35 @@ class TestClusterConductorWithK8s(base.TestCase):
     @patch('magnum.drivers.k8s_fedora_atomic_v1.driver.Driver.'
            '_extract_template_definition')
     @patch('magnum.common.clients.OpenStackClients')
+    @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     def test_update_stack(self,
+                          mock_objects_nodegroup_list,
+                          mock_objects_cluster_template_get_by_uuid,
                           mock_osc,
                           mock_extract_template_definition,
                           mock_get_template_contents):
 
         mock_stack_id = 'xx-xx-xx-xx'
-        mock_heat_client = mock.MagicMock()
+        mock_stack = mock.MagicMock(parameters={'number_of_minions': 1})
+        mock_stacks = mock.MagicMock()
+        mock_stacks.get.return_value = mock_stack
+        mock_heat_client = mock.MagicMock(stacks=mock_stacks)
         mock_osc.return_value.heat.return_value = mock_heat_client
-        mock_cluster = mock.MagicMock()
-        mock_cluster.stack_id = mock_stack_id
+        mock_template = objects.ClusterTemplate(
+            self.context, **self.cluster_template_dict)
+        mock_objects_cluster_template_get_by_uuid.return_value = mock_template
+        mock_cluster = objects.Cluster(self.context, **self.cluster_dict)
+        mock_cluster.cluster_template = mock_template
+        self.worker_ng_dict['node_count'] = 2
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
 
         k8s_dr.Driver().update_cluster({}, mock_cluster)
 
         expected_args = {
-            'parameters': {'number_of_minions': mock_cluster.node_count},
+            'parameters': {'number_of_minions': '2'},
             'existing': True,
             'disable_rollback': True
         }
