@@ -61,9 +61,6 @@ class TestClusterConductorWithMesos(base.TestCase):
             'name': 'cluster1',
             'stack_id': 'xx-xx-xx-xx',
             'api_address': '172.17.2.3',
-            'node_addresses': ['172.17.2.4'],
-            'node_count': 1,
-            'master_count': 1,
             'trustee_username': 'fake_trustee',
             'trustee_password': 'fake_trustee_password',
             'trustee_user_id': '7b489f04-b458-4541-8179-6a48a553e656',
@@ -75,6 +72,41 @@ class TestClusterConductorWithMesos(base.TestCase):
                        'mesos_slave_executor_env_variables': '{}',
                        'mesos_slave_work_dir': '/tmp/mesos/slave'
                        },
+            'fixed_network': '',
+            'fixed_subnet': '',
+            'floating_ip_enabled': False,
+        }
+        self.worker_ng_dict = {
+            'uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
+            'name': 'worker_ng',
+            'cluster_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
+            'project_id': 'project_id',
+            'docker_volume_size': 20,
+            'labels': self.cluster_dict['labels'],
+            'flavor_id': 'flavor_id',
+            'image_id': 'image_id',
+            'node_addresses': ['172.17.2.4'],
+            'node_count': 1,
+            'role': 'worker',
+            'max_nodes': 5,
+            'min_nodes': 1,
+            'is_default': True
+        }
+        self.master_ng_dict = {
+            'uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a54',
+            'name': 'master_ng',
+            'cluster_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
+            'project_id': 'project_id',
+            'docker_volume_size': 20,
+            'labels': self.cluster_dict['labels'],
+            'flavor_id': 'master_flavor_id',
+            'image_id': 'image_id',
+            'node_addresses': ['172.17.2.18'],
+            'node_count': 1,
+            'role': 'master',
+            'max_nodes': 5,
+            'min_nodes': 1,
+            'is_default': True
         }
         self.context.user_name = 'mesos_user'
         self.context.project_id = 'admin'
@@ -91,16 +123,21 @@ class TestClusterConductorWithMesos(base.TestCase):
         self.mock_osc.url_for.return_value = 'http://192.168.10.10:5000/v3'
 
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     def test_extract_template_definition_all_values(
             self,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid):
         cluster_template = objects.ClusterTemplate(
             self.context, **self.cluster_template_dict)
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = mesos_dr.Driver()
 
         (template_path,
@@ -150,10 +187,12 @@ class TestClusterConductorWithMesos(base.TestCase):
             env_files)
 
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     def test_extract_template_definition_only_required(
             self,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid):
         not_required = ['image_id', 'master_flavor_id', 'flavor_id',
                         'dns_nameserver', 'fixed_network', 'http_proxy',
@@ -167,6 +206,9 @@ class TestClusterConductorWithMesos(base.TestCase):
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = mesos_dr.Driver()
 
         (template_path,
@@ -208,12 +250,14 @@ class TestClusterConductorWithMesos(base.TestCase):
             env_files)
 
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.common.keystone.KeystoneClientV3')
     def test_extract_template_definition_with_lb_neutron(
             self,
             mock_kc,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid):
         self.cluster_template_dict['master_lb_enabled'] = True
         cluster_template = objects.ClusterTemplate(
@@ -221,6 +265,9 @@ class TestClusterConductorWithMesos(base.TestCase):
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = mesos_dr.Driver()
 
         mock_kc.return_value.client.services.list.return_value = []
@@ -272,12 +319,14 @@ class TestClusterConductorWithMesos(base.TestCase):
             env_files)
 
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.common.keystone.KeystoneClientV3')
     def test_extract_template_definition_with_lb_octavia(
             self,
             mock_kc,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid):
         self.cluster_template_dict['master_lb_enabled'] = True
         cluster_template = objects.ClusterTemplate(
@@ -285,6 +334,9 @@ class TestClusterConductorWithMesos(base.TestCase):
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = mesos_dr.Driver()
 
         class Service(object):
@@ -341,20 +393,25 @@ class TestClusterConductorWithMesos(base.TestCase):
             env_files)
 
     @patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    @patch('magnum.objects.NodeGroup.list')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
     @patch('magnum.common.keystone.KeystoneClientV3')
     def test_extract_template_definition_multi_master(
             self,
             mock_kc,
             mock_driver,
+            mock_objects_nodegroup_list,
             mock_objects_cluster_template_get_by_uuid):
         self.cluster_template_dict['master_lb_enabled'] = True
-        self.cluster_dict['master_count'] = 2
+        self.master_ng_dict['node_count'] = 2
         cluster_template = objects.ClusterTemplate(
             self.context, **self.cluster_template_dict)
         mock_objects_cluster_template_get_by_uuid.return_value = \
             cluster_template
         cluster = objects.Cluster(self.context, **self.cluster_dict)
+        worker_ng = objects.NodeGroup(self.context, **self.worker_ng_dict)
+        master_ng = objects.NodeGroup(self.context, **self.master_ng_dict)
+        mock_objects_nodegroup_list.return_value = [master_ng, worker_ng]
         mock_driver.return_value = mesos_dr.Driver()
 
         mock_kc.return_value.client.services.list.return_value = []
@@ -409,11 +466,23 @@ class TestClusterConductorWithMesos(base.TestCase):
     @patch('magnum.conf.CONF')
     @patch('magnum.common.clients.OpenStackClients')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
-    def setup_poll_test(self, mock_driver, mock_openstack_client, mock_conf,
-                        mock_retrieve_cluster_template):
+    def setup_poll_test(self, mock_driver, mock_openstack_client,
+                        mock_conf, mock_retrieve_cluster_template):
         mock_conf.cluster_heat.max_attempts = 10
 
-        cluster = mock.MagicMock()
+        worker_ng = mock.MagicMock(
+            uuid='5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
+            role='worker',
+            node_count=1,
+        )
+        master_ng = mock.MagicMock(
+            uuid='5d12f6fd-a196-4bf0-ae4c-1f639a523a54',
+            role='master',
+            node_count=1,
+        )
+        cluster = mock.MagicMock(nodegroups=[worker_ng, master_ng],
+                                 default_ng_worker=worker_ng,
+                                 default_ng_master=master_ng)
         mock_heat_stack = mock.MagicMock()
         mock_heat_client = mock.MagicMock()
         mock_heat_client.stacks.get.return_value = mock_heat_stack
@@ -425,23 +494,30 @@ class TestClusterConductorWithMesos(base.TestCase):
         poller = heat_driver.HeatPoller(mock_openstack_client,
                                         mock.MagicMock(), cluster,
                                         mesos_dr.Driver())
+        poller.template_def.add_nodegroup_params(cluster)
         poller.get_version_info = mock.MagicMock()
         return (mock_heat_stack, cluster, poller)
 
     def test_poll_node_count(self):
         mock_heat_stack, cluster, poller = self.setup_poll_test()
 
-        mock_heat_stack.parameters = {'number_of_slaves': 1}
+        mock_heat_stack.parameters = {
+            'number_of_slaves': 1,
+            'number_of_masters': 1
+        }
         mock_heat_stack.stack_status = cluster_status.CREATE_IN_PROGRESS
         poller.poll_and_check()
 
-        self.assertEqual(1, cluster.node_count)
+        self.assertEqual(1, cluster.default_ng_worker.node_count)
 
     def test_poll_node_count_by_update(self):
         mock_heat_stack, cluster, poller = self.setup_poll_test()
 
-        mock_heat_stack.parameters = {'number_of_slaves': 2}
+        mock_heat_stack.parameters = {
+            'number_of_slaves': 2,
+            'number_of_masters': 1
+        }
         mock_heat_stack.stack_status = cluster_status.UPDATE_COMPLETE
         poller.poll_and_check()
 
-        self.assertEqual(2, cluster.node_count)
+        self.assertEqual(2, cluster.default_ng_worker.node_count)
