@@ -13,16 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections import namedtuple
+
 import mock
 from oslo_serialization import jsonutils
 
+from magnum.common import exception
 from magnum.drivers.common import k8s_monitor
 from magnum.drivers.mesos_ubuntu_v1 import monitor as mesos_monitor
 from magnum.drivers.swarm_fedora_atomic_v1 import monitor as swarm_monitor
+from magnum.drivers.swarm_fedora_atomic_v2 import monitor as swarm_v2_monitor
 from magnum import objects
+from magnum.objects import fields as m_fields
 from magnum.tests import base
 from magnum.tests.unit.db import utils
 
+NODE_STATUS_CONDITION = namedtuple('Condition',
+                                   ['type', 'status'])
+
 
 class MonitorsTestCase(base.TestCase):
 
@@ -44,7 +52,15 @@ class MonitorsTestCase(base.TestCase):
                                          api_address='https://5.6.7.8:2376',
                                          master_addresses=['10.0.0.6'])
         self.cluster = objects.Cluster(self.context, **cluster)
+        nodegroups = utils.get_nodegroups_for_cluster(
+            node_addresses=['1.2.3.4'], master_addresses=['10.0.0.6'])
+        self.nodegroups = [
+            objects.NodeGroup(self.context, **nodegroups['master']),
+            objects.NodeGroup(self.context, **nodegroups['worker'])
+        ]
         self.monitor = swarm_monitor.SwarmMonitor(self.context, self.cluster)
+        self.v2_monitor = swarm_v2_monitor.SwarmMonitor(self.context,
+                                                        self.cluster)
         self.k8s_monitor = k8s_monitor.K8sMonitor(self.context, self.cluster)
         self.mesos_monitor = mesos_monitor.MesosMonitor(self.context,
                                                         self.cluster)
@@ -55,6 +71,13 @@ class MonitorsTestCase(base.TestCase):
         self.mock_metrics_spec.return_value = self.test_metrics_spec
         self.addCleanup(p.stop)
 
+        p2 = mock.patch('magnum.drivers.swarm_fedora_atomic_v2.monitor.'
+                        'SwarmMonitor.metrics_spec',
+                        new_callable=mock.PropertyMock)
+        self.mock_metrics_spec_v2 = p2.start()
+        self.mock_metrics_spec_v2.return_value = self.test_metrics_spec
+        self.addCleanup(p2.stop)
+
     @mock.patch('magnum.common.docker_utils.docker_for_cluster')
     def test_swarm_monitor_pull_data_success(self, mock_docker_cluster):
         mock_docker = mock.MagicMock()
@@ -70,6 +93,22 @@ class MonitorsTestCase(base.TestCase):
                          self.monitor.data['nodes'])
         self.assertEqual(['test_container'], self.monitor.data['containers'])
 
+    @mock.patch('magnum.common.docker_utils.docker_for_cluster')
+    def test_swarm_v2_monitor_pull_data_success(self, mock_docker_cluster):
+        mock_docker = mock.MagicMock()
+        mock_docker.info.return_value = {'DriverStatus': [[
+            u' \u2514 Reserved Memory', u'0 B / 1 GiB']]}
+        mock_docker.containers.return_value = [mock.MagicMock()]
+        mock_docker.inspect_container.return_value = 'test_container'
+        mock_docker_cluster.return_value.__enter__.return_value = mock_docker
+
+        self.v2_monitor.pull_data()
+
+        self.assertEqual([{'MemTotal': 1073741824.0}],
+                         self.v2_monitor.data['nodes'])
+        self.assertEqual(['test_container'],
+                         self.v2_monitor.data['containers'])
+
     @mock.patch('magnum.common.docker_utils.docker_for_cluster')
     def test_swarm_monitor_pull_data_raise(self, mock_docker_cluster):
         mock_container = mock.MagicMock()
@@ -86,14 +125,38 @@ class MonitorsTestCase(base.TestCase):
                          self.monitor.data['nodes'])
         self.assertEqual([mock_container], self.monitor.data['containers'])
 
+    @mock.patch('magnum.common.docker_utils.docker_for_cluster')
+    def test_swarm_v2_monitor_pull_data_raise(self, mock_docker_cluster):
+        mock_container = mock.MagicMock()
+        mock_docker = mock.MagicMock()
+        mock_docker.info.return_value = {'DriverStatus': [[
+            u' \u2514 Reserved Memory', u'0 B / 1 GiB']]}
+        mock_docker.containers.return_value = [mock_container]
+        mock_docker.inspect_container.side_effect = Exception("inspect error")
+        mock_docker_cluster.return_value.__enter__.return_value = mock_docker
+
+        self.v2_monitor.pull_data()
+
+        self.assertEqual([{'MemTotal': 1073741824.0}],
+                         self.v2_monitor.data['nodes'])
+        self.assertEqual([mock_container], self.v2_monitor.data['containers'])
+
     def test_swarm_monitor_get_metric_names(self):
         names = self.monitor.get_metric_names()
         self.assertEqual(sorted(['metric1', 'metric2']), sorted(names))
 
+    def test_swarm_v2_monitor_get_metric_names(self):
+        names = self.v2_monitor.get_metric_names()
+        self.assertEqual(sorted(['metric1', 'metric2']), sorted(names))
+
     def test_swarm_monitor_get_metric_unit(self):
         unit = self.monitor.get_metric_unit('metric1')
         self.assertEqual('metric1_unit', unit)
 
+    def test_swarm_v2_monitor_get_metric_unit(self):
+        unit = self.v2_monitor.get_metric_unit('metric1')
+        self.assertEqual('metric1_unit', unit)
+
     def test_swarm_monitor_compute_metric_value(self):
         mock_func = mock.MagicMock()
         mock_func.return_value = 'metric1_value'
@@ -101,6 +164,13 @@ class MonitorsTestCase(base.TestCase):
         value = self.monitor.compute_metric_value('metric1')
         self.assertEqual('metric1_value', value)
 
+    def test_swarm_v2_monitor_compute_metric_value(self):
+        mock_func = mock.MagicMock()
+        mock_func.return_value = 'metric1_value'
+        self.v2_monitor.metric1_func = mock_func
+        value = self.v2_monitor.compute_metric_value('metric1')
+        self.assertEqual('metric1_value', value)
+
     def test_swarm_monitor_compute_memory_util(self):
         test_data = {
             'nodes': [
@@ -130,6 +200,35 @@ class MonitorsTestCase(base.TestCase):
         mem_util = self.monitor.compute_memory_util()
         self.assertEqual(0, mem_util)
 
+    def test_swarm_v2_monitor_compute_memory_util(self):
+        test_data = {
+            'nodes': [
+                {
+                    'Name': 'node',
+                    'MemTotal': 20,
+                },
+            ],
+            'containers': [
+                {
+                    'Name': 'container',
+                    'HostConfig': {
+                        'Memory': 10,
+                    },
+                },
+            ],
+        }
+        self.v2_monitor.data = test_data
+        mem_util = self.v2_monitor.compute_memory_util()
+        self.assertEqual(50, mem_util)
+
+        test_data = {
+            'nodes': [],
+            'containers': [],
+        }
+        self.v2_monitor.data = test_data
+        mem_util = self.v2_monitor.compute_memory_util()
+        self.assertEqual(0, mem_util)
+
     @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
     def test_k8s_monitor_pull_data_success(self, mock_k8s_api):
         mock_nodes = mock.MagicMock()
@@ -238,8 +337,11 @@ class MonitorsTestCase(base.TestCase):
         self.assertEqual(self.mesos_monitor.data['cpu_used'],
                          expected_cpu_used)
 
+    @mock.patch('magnum.objects.NodeGroup.list')
     @mock.patch('magnum.common.urlfetch.get')
-    def test_mesos_monitor_pull_data_success(self, mock_url_get):
+    def test_mesos_monitor_pull_data_success(self, mock_url_get,
+                                             mock_ng_list):
+        mock_ng_list.return_value = self.nodegroups
         state_json = {
             'leader': 'master@10.0.0.6:5050',
             'pid': 'master@10.0.0.6:5050',
@@ -257,8 +359,11 @@ class MonitorsTestCase(base.TestCase):
         self._test_mesos_monitor_pull_data(mock_url_get, state_json,
                                            100, 50, 1, 0.2)
 
+    @mock.patch('magnum.objects.NodeGroup.list')
     @mock.patch('magnum.common.urlfetch.get')
-    def test_mesos_monitor_pull_data_success_not_leader(self, mock_url_get):
+    def test_mesos_monitor_pull_data_success_not_leader(self, mock_url_get,
+                                                        mock_ng_list):
+        mock_ng_list.return_value = self.nodegroups
         state_json = {
             'leader': 'master@10.0.0.6:5050',
             'pid': 'master@1.1.1.1:5050',
@@ -267,9 +372,11 @@ class MonitorsTestCase(base.TestCase):
         self._test_mesos_monitor_pull_data(mock_url_get, state_json,
                                            0, 0, 0, 0)
 
+    @mock.patch('magnum.objects.NodeGroup.list')
     @mock.patch('magnum.common.urlfetch.get')
-    def test_mesos_monitor_pull_data_success_no_master(self, mock_url_get):
-        self.cluster.master_addresses = []
+    def test_mesos_monitor_pull_data_success_no_master(self, mock_url_get,
+                                                       mock_ng_list):
+        mock_ng_list.return_value = []
         self._test_mesos_monitor_pull_data(mock_url_get, {}, 0, 0, 0, 0)
 
     def test_mesos_monitor_get_metric_names(self):
@@ -332,3 +439,75 @@ class MonitorsTestCase(base.TestCase):
         self.mesos_monitor.data = test_data
         cpu_util = self.mesos_monitor.compute_cpu_util()
         self.assertEqual(0, cpu_util)
+
+    @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
+    def test_k8s_monitor_health_healthy(self, mock_k8s_api):
+        mock_nodes = mock.MagicMock()
+        mock_node = mock.MagicMock()
+        mock_api_client = mock.MagicMock()
+        mock_node.status = mock.MagicMock()
+        mock_node.metadata.name = 'k8s-cluster-node-0'
+        mock_node.status.conditions = [NODE_STATUS_CONDITION(type='Ready',
+                                                             status='True')]
+        mock_nodes.items = [mock_node]
+        mock_k8s_api.return_value.list_node.return_value = (
+            mock_nodes)
+        mock_k8s_api.return_value.api_client = mock_api_client
+        mock_api_client.call_api.return_value = ('ok', None, None)
+
+        self.k8s_monitor.poll_health_status()
+        self.assertEqual(self.k8s_monitor.data['health_status'],
+                         m_fields.ClusterHealthStatus.HEALTHY)
+        self.assertEqual(self.k8s_monitor.data['health_status_reason'],
+                         {'api': 'ok', 'k8s-cluster-node-0.Ready': True})
+
+    @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
+    def test_k8s_monitor_health_unhealthy_api(self, mock_k8s_api):
+        mock_nodes = mock.MagicMock()
+        mock_node = mock.MagicMock()
+        mock_api_client = mock.MagicMock()
+        mock_node.status = mock.MagicMock()
+        mock_node.metadata.name = 'k8s-cluster-node-0'
+        mock_node.status.conditions = [NODE_STATUS_CONDITION(type='Ready',
+                                                             status='True')]
+        mock_nodes.items = [mock_node]
+        mock_k8s_api.return_value.list_node.return_value = (
+            mock_nodes)
+        mock_k8s_api.return_value.api_client = mock_api_client
+        mock_api_client.call_api.side_effect = exception.MagnumException(
+            message='failed')
+
+        self.k8s_monitor.poll_health_status()
+        self.assertEqual(self.k8s_monitor.data['health_status'],
+                         m_fields.ClusterHealthStatus.UNHEALTHY)
+        self.assertEqual(self.k8s_monitor.data['health_status_reason'],
+                         {'api': 'failed'})
+
+    @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
+    def test_k8s_monitor_health_unhealthy_node(self, mock_k8s_api):
+        mock_nodes = mock.MagicMock()
+        mock_api_client = mock.MagicMock()
+
+        mock_node0 = mock.MagicMock()
+        mock_node0.status = mock.MagicMock()
+        mock_node0.metadata.name = 'k8s-cluster-node-0'
+        mock_node0.status.conditions = [NODE_STATUS_CONDITION(type='Ready',
+                                                              status='False')]
+        mock_node1 = mock.MagicMock()
+        mock_node1.status = mock.MagicMock()
+        mock_node1.metadata.name = 'k8s-cluster-node-1'
+        mock_node1.status.conditions = [NODE_STATUS_CONDITION(type='Ready',
+                                                              status='True')]
+
+        mock_nodes.items = [mock_node0, mock_node1]
+        mock_k8s_api.return_value.list_node.return_value = (
+            mock_nodes)
+        mock_k8s_api.return_value.api_client = mock_api_client
+        mock_api_client.call_api.return_value = ('ok', None, None)
+
+        self.k8s_monitor.poll_health_status()
+        self.assertEqual(self.k8s_monitor.data['health_status'],
+                         m_fields.ClusterHealthStatus.UNHEALTHY)
+        self.assertEqual(self.k8s_monitor.data['health_status_reason'],
+                         {'api': 'ok', 'k8s-cluster-node-0.Ready': False,
+                          'api': 'ok', 'k8s-cluster-node-1.Ready': True})
