@@ -14,6 +14,7 @@
 #    under the License.
 """Magnum test utilities."""
 
+from oslo_utils import uuidutils
 
 from magnum.db import api as db_api
 
@@ -91,23 +92,33 @@ def get_test_cluster(**kw):
         'status_reason': kw.get('status_reason', 'Completed successfully'),
         'create_timeout': kw.get('create_timeout', 60),
         'api_address': kw.get('api_address', '172.17.2.3'),
-        'node_addresses': kw.get('node_addresses', ['172.17.2.4']),
-        'node_count': kw.get('node_count', 3),
-        'master_count': kw.get('master_count', 3),
-        'master_addresses': kw.get('master_addresses', ['172.17.2.18']),
         'created_at': kw.get('created_at'),
         'updated_at': kw.get('updated_at'),
         'docker_volume_size': kw.get('docker_volume_size'),
         'labels': kw.get('labels'),
         'master_flavor_id': kw.get('master_flavor_id', None),
         'flavor_id': kw.get('flavor_id', None),
+        'fixed_network': kw.get('fixed_network', None),
+        'fixed_subnet': kw.get('fixed_subnet', None),
+        'floating_ip_enabled': kw.get('floating_ip_enabled', True),
     }
 
+    if kw.pop('for_api_use', False):
+        attrs.update({
+            'node_addresses': kw.get('node_addresses', ['172.17.2.4']),
+            'node_count': kw.get('node_count', 3),
+            'master_count': kw.get('master_count', 3),
+            'master_addresses': kw.get('master_addresses', ['172.17.2.18'])
+        })
     # Only add Keystone trusts related attributes on demand since they may
     # break other tests.
     for attr in ['trustee_username', 'trustee_password', 'trust_id']:
         if attr in kw:
             attrs[attr] = kw[attr]
+    # Required only in PeriodicTestCase, may break other tests
+    for attr in ['keypair', 'health_status']:
+        if attr in kw:
+            attrs[attr] = kw[attr]
 
     return attrs
 
@@ -267,3 +278,80 @@ def create_test_federation(**kw):
         del federation['id']
     dbapi = db_api.get_instance()
     return dbapi.create_federation(federation)
+
+
+def get_test_nodegroup(**kw):
+    return {
+        'id': kw.get('id', 12),
+        'uuid': kw.get('uuid', '483203a3-dbee-4a9c-9d65-9820512f4df8'),
+        'name': kw.get('name', 'nodegroup1'),
+        'cluster_id': kw.get('cluster_id',
+                             '5d12f6fd-a196-4bf0-ae4c-1f639a523a52'),
+        'project_id': kw.get('project_id', 'fake_project'),
+        'docker_volume_size': kw.get('docker_volume_size'),
+        'labels': kw.get('labels'),
+        'flavor_id': kw.get('flavor_id', None),
+        'image_id': kw.get('image_id', None),
+        'node_addresses': kw.get('node_addresses', ['172.17.2.4']),
+        'node_count': kw.get('node_count', 3),
+        'role': kw.get('role', 'worker'),
+        'max_node_count': kw.get('max_node_count', None),
+        'min_node_count': kw.get('min_node_count', 1),
+        'is_default': kw.get('is_default', True),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at')
+    }
+
+
+def create_test_nodegroup(**kw):
+    """Create test nodegroup entry in DB and return federation DB object.
+
+    :param kw: kwargs with overriding values for nodegroup attributes.
+    :return: Test nodegroup DB object.
+    """
+    nodegroup = get_test_nodegroup(**kw)
+    # Let DB generate ID if it isn't specified explicitly
+    if 'id' in nodegroup:
+        del nodegroup['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_nodegroup(nodegroup)
+
+
+def get_nodegroups_for_cluster(**kw):
+    # get workers nodegroup
+    worker = get_test_nodegroup(
+        role='worker',
+        name=kw.get('worker_name', 'test-worker'),
+        uuid=kw.get('worker_uuid', uuidutils.generate_uuid()),
+        cluster_id=kw.get('cluster_id',
+                          '5d12f6fd-a196-4bf0-ae4c-1f639a523a52'),
+        project_id=kw.get('project_id', 'fake_project'),
+        node_addresses=kw.get('node_addresses', ['172.17.2.4']),
+        node_count=kw.get('node_count', 3)
+    )
+
+    # get masters nodegroup
+    master = get_test_nodegroup(
+        role='master',
+        name=kw.get('master_name', 'test-master'),
+        uuid=kw.get('master_uuid', uuidutils.generate_uuid()),
+        cluster_id=kw.get('cluster_id',
+                          '5d12f6fd-a196-4bf0-ae4c-1f639a523a52'),
+        project_id=kw.get('project_id', 'fake_project'),
+        node_addresses=kw.get('master_addresses', ['172.17.2.18']),
+        node_count=kw.get('master_count', 3)
+    )
+    return {'master': master, 'worker': worker}
+
+
+def create_nodegroups_for_cluster(**kw):
+    nodegroups = get_nodegroups_for_cluster(**kw)
+    # Create workers nodegroup
+    worker = nodegroups['worker']
+    del worker['id']
+    create_test_nodegroup(**worker)
+
+    # Create masters nodegroup
+    master = nodegroups['master']
+    del master['id']
+    create_test_nodegroup(**master)
